
import os
import shutil

from twisted.internet.defer import inlineCallbacks, returnValue
from twisted.web.client import downloadPage
from twisted.web.error import Error

from ensemble.errors import FileNotFound
from ensemble.formula.bundle import FormulaBundle

from ensemble.state.formula import FormulaStateManager


@inlineCallbacks
def download_formula(client, formula_id, formulas_directory):
    formula_state_manager = FormulaStateManager(client)
    formula_state = yield formula_state_manager.get_formula_state(formula_id)

    # Calculate local formula path
    checksum = yield formula_state.get_sha256()
    formula_key = "%s:%s" % (formula_state.id, checksum)
    local_formula_path = os.path.join(
        formulas_directory, formula_key)

    # Retrieve formula from provider storage link
    if formula_state.bundle_url.startswith("file://"):
        file_path = formula_state.bundle_url[len("file://"):]
        if not os.path.exists(file_path):
            raise FileNotFound(formula_state.bundle_url)
        shutil.copyfileobj(open(file_path), open(local_formula_path, "w"))
    else:
        try:
            yield downloadPage(formula_state.bundle_url, local_formula_path)
        except Error:
            raise FileNotFound(formula_state.bundle_url)
    returnValue(FormulaBundle(local_formula_path))
